/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
  ThinkGeek decoder: Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>
  If you happen to buy the "you are dumb" or the "you are dumb v2.0" of
  the ThinkGeek T-shirts, you might want to prove their correctness.
  So, here is a decoder, which takes binary secuences and translates
  them to ascii text. You might find another use for it too!
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXLEN 10000
   
void introduce_yourself(void);
void take_sequence(void);
void split_sequence_in_octets(void);
int calculate_decimal_checksum_of_octet(int k);
char find_ascii_equivalent(int checksum);
void add_ascii_letter_to_string(char ascii_char);
	
char bin_seq[MAXLEN];
int octet[MAXLEN/8][8];    
int checksum = 0;
char ascii_char;
char msg[MAXLEN/8] = "";

int main(void)
{
    
  int k,j;

  introduce_yourself();
  take_sequence();
  
  printf("Ok, you gave me the following valid binary sequence:\n");
  printf("%s\n",bin_seq);
  printf("Now, watch me doing the translation...\n");
  printf("\n");
  printf("octet\t\tdecimal checksum\tascii equivalent\n");
	 
  split_sequence_in_octets();
  
  for(k=0;k<strlen(bin_seq)/8;k++){
    
    checksum = calculate_decimal_checksum_of_octet(k);
    ascii_char = find_ascii_equivalent(checksum);
    add_ascii_letter_to_string(ascii_char);

    for(j=0;j<8;j++)
      printf("%d",octet[k][j]);

    printf("\t");
    printf("%d\t\t\t%c\n",checksum,ascii_char);

    }

  printf("\n");
  printf("And the whole message in ascii text is...:\n");
  printf("%s\n",msg);



}

void introduce_yourself(void)
{
    printf("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. tgdecoder is licensed under the GNU General Public License. See the file COPYING for details.\n\n");
    printf("Copyright (C) 2001 Thomas Repantis <darkzero@otenet.gr>\n");
    printf("Hello, this is the ThinkGeek decoder!\n");
    printf("My purpose in life is to translate binary sequences to ascii text.\n");
}

void take_sequence(void)
{

  int i;
  int err_flag=0;

  printf("Give me the binary sequence you have.\n");
  printf(">");
  scanf("%s",bin_seq);


  /*Error checking.*/

  for(i=0;i<strlen(bin_seq);i++){
    if(bin_seq[i]!='0'&&bin_seq[i]!='1'){
      printf("Hey, are you joking? Binary sequences consist only of 0 and 1!\n");
      err_flag=1;
      break;
    }
  }

  if((strlen(bin_seq)%8)!=0){
    printf("Hey, this binary sequence isn't a multiple of 8! How do you expect me to find its ascii text equivalent?\n");
    err_flag=1;
  }

  if(strlen(bin_seq)>MAXLEN){
    printf("Hey, are you trying to overflow me? The binary sequence must be smaller than %d digits!\n", MAXLEN);
    err_flag=1;
  }

  if(err_flag!=0)
    exit(1);

}



void split_sequence_in_octets(void)
{

  int i=0,j,k;

  for(k=0;k<strlen(bin_seq)/8;k++)
    for(j=0;j<8;j++,i++)
      octet[k][j]=bin_seq[i]-'0';
  
}


int calculate_decimal_checksum_of_octet(int k)
{
  checksum = 128*octet[k][0] + 64*octet[k][1] + 32*octet[k][2] + 16*octet[k][3]
    + 8*octet[k][4] + 4*octet[k][5] + 2*octet[k][6] + 1*octet[k][7];
  return checksum;
}

char find_ascii_equivalent(int checksum)
{

  ascii_char = checksum;
  return ascii_char;

}

void add_ascii_letter_to_string(char ascii_char)
{

  char letter[1];
  letter[0]=ascii_char;
  letter[1]='\0';
  strcat(msg,letter);

}
	








